/*
 * Created on Aug 31, 2007
 *
 * Copyright (c) 2007, the JUNG Project and the Regents of the University 
 * of California
 * All rights reserved.
 *
 * This software is open-source under the BSD license; see either
 * "license.txt" or
 * http://jung.sourceforge.net/license.txt for a description.
 */
package edu.uci.ics.jung.graph;

/**
 * A tagging interface which indicates that the implementing graph accepts
 * parallel edges.
 * 
 * @author Joshua O'Madadhain
 */
public interface MultiGraph<V, E> {}
