package geogebra.kernel.kernelND;

/**
 * Interface to set the level of detail of surface, curve, etc.
 * @author matthieu
 *
 */
public interface LevelOfDetail {

	public void setLevelOfDetail(int val);
	
	public int getLevelOfDetail();
	
	
}
