package geogebra.gui.view.spreadsheet.statdialog;

import java.awt.Font;

public interface StatPanelInterface {

	public void updateFonts(Font font);
	
	public void setLabels();
	
	public void updatePanel();
	
}
