package geogebra.kernel.discrete.tsp;

public class Order {
	public static void main(String[] args) {
		int n = 150;
		double num = 1;
		for (int i = n; i > 2; i--) {
			num *= i;
		}
		//System.out.println(num);
	}
}
