package geogebra.gui.menubar;

public interface Menubar {
	public void initMenubar();
	public void updateMenubar();
	public void updateMenuFile();
	public void updateMenuWindow();
	public void updateSelection();
	public void updateActions();
}