package geogebra.euclidian;

public class FormulaDimension {
	
	public int width, height;
	
	// depth not working eg woth TableText
	public int depth = 0;

	public FormulaDimension(int width, int height){//, int depth) {
		this.width = width;
		this.height = height;
		//this.depth = depth;
	}

	public FormulaDimension() {
		// TODO Auto-generated constructor stub
	}

}
