package geogebra.euclidian;

import java.awt.Color;

public interface PropertiesPanelMiniListener {
	
	public void setLineStyle(int lineStyle);
	public void setSize(int size);
	public void setColor(Color color);

}
