package geogebra.main.settings;

import java.util.LinkedList;

/**
 * Settings for the algebra view.
 */
public class AlgebraSettings extends AbstractSettings {

	public AlgebraSettings(LinkedList<SettingListener> listeners) {
		super(listeners);
	}

	public AlgebraSettings() {
		super();
	}
}
