package geogebra.gui.util;

/**
 * Interface for SpecialNumberFormat. 
 */
public interface SpecialNumberFormatInterface {

	/**
	 * Callback method used by SpecialNumberFormat's action listener whenever a
	 * number format has changed.
	 */
	public void changedNumberFormat();
	
}
