package geogebra.kernel;

public class CircularDefinitionException extends Exception {

	private static final long serialVersionUID = -1012964850092798314L;
	
	public CircularDefinitionException() {
		super("CircularDefinition");
	}
	
}
