/*
 * Created on February 27, 2007
 *
 * Copyright (c) 2007, the JUNG Project and the Regents of the University 
 * of California
 * All rights reserved.
 *
 * This software is open-source under the BSD license; see either
 * "license.txt" or
 * http://jung.sourceforge.net/license.txt for a description.
 */

package edu.uci.ics.jung.graph.util;

/**
 * Defines the possible edge types for graphs which assign types to edges.
 */
public enum EdgeType 
{
      DIRECTED,
      UNDIRECTED
}
