package geogebra.kernel.discrete.signalprocesser.voronoi.shapegeneration;

public class ShapeGenerationException extends Exception {
    
    public ShapeGenerationException(String message) {
        super(message);
    }
    
}
